/*  S-Lang console functions
 *
 *  (C) Copyright 2004-2006 Denis Rojo <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __CONSOLE_H__
#define __CONSOLE_H__

#include <inttypes.h>
#include <stdarg.h>
#include <slang.h>


extern volatile int SLang_Error; // hack for old versions


#define CHAR_BLANK 0x20

#define KEY_ENTER 13
#define KEY_SPACE 32
#define KEY_BACKSPACE 275
#define KEY_BACKSPACE_APPLE 127 
#define KEY_BACKSPACE_SOMETIMES 272
#define KEY_LEFT 259
#define KEY_RIGHT 260
#define KEY_HOME 263
#define KEY_DELETE 275
#define KEY_TAB 9

/* unix ctrl- commandline hotkeys */
#define KEY_CTRL_A 1 // goto beginning of line
#define KEY_CTRL_B 2 // change blit
#define KEY_CTRL_D 4 // delete char
#define KEY_CTRL_E 5 // add new effect
#define KEY_CTRL_F 6 // go fullscreen
#define KEY_CTRL_G 7
#define KEY_CTRL_H_APPLE 8 // ctrl-h on apple/OSX
#define KEY_CTRL_I 9 // OSD on/off
#define KEY_CTRL_K 11 // delete until end of line
#define KEY_CTRL_L 12 // refresh screen
#define KEY_CTRL_M 13 // move layer
#define KEY_CTRL_U 21 // delete until beginning of line
#define KEY_CTRL_H 272 // help the user
#define KEY_CTRL_J 10 // javascript command
#define KEY_CTRL_O 15 // open a file in a new layer
#define KEY_CTRL_S 19 // start streaming (overrides scroll lock)
#define KEY_CTRL_T 20 // new layer with text
#define KEY_CTRL_V 22 // change blit value
#define KEY_CTRL_W 23 // start stream and save to file
#define KEY_CTRL_X 24
#define KEY_CTRL_Y 25

#define KEY_PLUS 43
#define KEY_MINUS 45



class Console {
 public:
  
  Console();
  ~Console();

  bool init();

  void close();

  uint64_t getkey(); ///< returns the key typed
  
  void putkey(uint64_t key); ///< interprets and print the keycode

  /** by short circuiting the above functions we already have a
      text friendly environment where to move and write around */

  bool refresh(); ///< run one cycle, return false if quit signal was received

  ////////////////////////
  /// messaging interface
  void notice(char *str);
  void act(char *str);
  void warning(char *str);
  void error(char *str);
  void func(char *str);
  ////////////////////////

  /** by now these are simple coordinates of a full screen text editor
      a layered environment of text windows and some widgets would be
      nice to have. i'm planning that if the ram of the z50 holds */
  int w, h;
  int x, y;

  bool wrap;

 private:
  // double buffer
  char *buffer;

  void scroll_down(int y, int len);
  void clean_overlap(int dest_off, int src_off, int len);
  void move(int dest_x, int dest_y, int src_x, int src_y, int len);
  void putch(char ch);

};

#endif

