/*  Time Based Text - Recorder
 *
 *  (C) Copyright 2006 Denis Rojo <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>

#include <getopt.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>

#include <console.h>
#include <tbt.h>
#include <jutils.h>

static const char *help =
"Usage: rectext [options] [file]\n"
"\n"
"  -h   print this help\n"
"  -v   version information\n"
"  -D   debug verbosity level - default 1\n"
"  -r   render format (bin|ascii|javascript)\n";
// "  -w   word wrap on screen bounds\n"
// "  -s   scrolling text\n"

static const char *short_options = "-hvD:r:";

int debug;
char filename[512];

// rendering formats
#define BIN        1
#define ASCII      2
#define JS         3
int render = BIN;

void cmdline(int argc, char **argv) {
  int res;

  debug = 1;
  filename[0] = 0x0;

  do {
    res = getopt(argc, argv, short_options);
    switch(res) {
    case 'h':
      fprintf(stderr, "rectext:\t");
      fprintf(stderr, VERSION);
      fprintf(stderr, "%s", help);
      exit(0);
      break;

    case 'v':
      fprintf(stderr,"\n");
      exit(0);
      break;

    case 'D':
      debug = atoi(optarg);
      if(debug>3) {
	warning("debug verbosity ranges from 1 to 3\n");
	debug = 3;
      }
      break;

    case 'r':
      if( strncasecmp(optarg, "BIN", 3) ==0)
	      render = BIN;
      else if( strncasecmp(optarg, "ASCII", 5) ==0)
	      render = ASCII;
      else if( strncasecmp(optarg, "JAVASCRIPT", 10) ==0)
	      render = JS;
      else {
	      error ("render format not recognized: %s", optarg);
	      act ("using default binary format render");
	      render = BIN;
      }
      break;
	      

    case 1:
      {
	FILE *fd;
	snprintf(filename,511,"%s", optarg);
	fd = fopen(filename, "w");
	if(!fd) {
	  error("can't write in %s: %s", filename, strerror(errno));
	  exit(1);
	} else fclose(fd);
      }
      
    default: break;
      
    }
  } while (res != -1);

#ifdef HAVE_BSD
  for(;optind<argc;optind++)
    snprintf(filename, 511, "%s", argv[optind]);
#endif

  if(!filename[0])
    sprintf(filename, "/tmp/record.tbt");

  set_debug(debug);

}

int main(int argc, char** argv)
{

  cmdline(argc, argv);

  // initialize the text console
  Console con;
  if(! con.init() ) exit(-1);
  
  // initialize the Time Based Text object
  TBT tbt;

  bool quit = false;
  
  while(!quit) {

    int key = con.getkey();

    if(key) {

      // display the key
      con.putkey(key);

      // save the key and timestamp
      tbt.append(key);

    }
    
    if( ! con.refresh() ) quit = true;

    jsleep(0,10);

  }

  if( render == BIN )
	  
  	tbt.save_bin( filename );

  else if( render == ASCII )

  	tbt.save_ascii( filename );

  else if( render == JS )

  	tbt.save_javascript( filename );
  
  con.close();

  exit(0);  
}
